#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# 2019-2020 Darko Poljak (darko.poljak at gmail.com)
#
# This file is part of cdist.
#
# cdist is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# cdist is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with cdist. If not, see <http://www.gnu.org/licenses/>.
#
#

import cdist
import cdist.configuration
import cdist.core
import cdist.exec.util as util
import os
import glob
import fnmatch


class Info(object):

    def __init__(self, conf_dirs, args):
        self.conf_dirs = conf_dirs
        self.all = args.all
        self.display_global_explorers = args.global_explorers
        self.display_types = args.types
        if not self.display_global_explorers and not self.display_types:
            self.all = True
        if args.pattern is None:
            self.glob_pattern = '*'
        else:
            self.glob_pattern = args.pattern
        self.display_headers = not args.suppress_headers

    @classmethod
    def commandline(cls, args):
        cfg = cdist.configuration.Configuration(args)
        configuration = cfg.get_config(section='GLOBAL')
        conf_dirs = util.resolve_conf_dirs(configuration,
                                           args.conf_dir)
        c = cls(conf_dirs, args)
        c.run()

    def _print_line(self):
        print('-' * 78)

    def _print_double_line(self):
        print('=' * 78)

    def _display_global_explorers_info(self, conf_path):
        if self.display_headers:
            print("{} config directory global explorers:".format(conf_path))
            self._print_double_line()
        global_explorer_path = os.path.join(conf_path, "explorer",
                                            self.glob_pattern)
        for explorer in glob.glob(global_explorer_path):
            print(explorer)
        if self.display_headers:
            self._print_line()

    def _should_display_type(self, dir_entry):
        if not dir_entry.is_dir():
            return False
        if self.glob_pattern is None:
            return True
        return fnmatch.fnmatch(dir_entry.name, self.glob_pattern)

    def _display_types_info(self, conf_path):
        if self.display_headers:
            print("{} config directory types:".format(conf_path))
            self._print_double_line()
        types_path = os.path.join(conf_path, "type")
        if not os.path.exists(types_path):
            if self.display_headers:
                self._print_line()
            return
        with os.scandir(types_path) as it:
            for entry in it:
                if self._should_display_type(entry):
                    print(entry.path)
        if self.display_headers:
            self._print_line()

    def _display_conf_dirs(self):
        print("Config directories:")
        self._print_double_line()
        print("{}".format('\n'.join(self.conf_dirs)))
        self._print_line()

    def run(self):
        if self.display_headers:
            self._display_conf_dirs()
        for conf_path in self.conf_dirs:
            if self.all or self.display_global_explorers:
                self._display_global_explorers_info(conf_path)
            if self.all or self.display_types:
                self._display_types_info(conf_path)
